/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/


#include "discover.h"
#include "cl_settings.h"

Cdiscover::Cdiscover(MApp_context& Context) : CCheckedActive(CCheckedActive::EPriorityIdle, _L("Cdiscover")), 
Mlog_base_impl(Context),
current_state(IDLE)
{
	CALLSTACKITEM(_L("Cdiscover::Cdiscover"));

}

void Cdiscover::ConstructL() 
{
	CALLSTACKITEM(_L("Cdiscover::ConstructL"));

	Mlog_base_impl::ConstructL(KLog_bt, _L(""));

	CActiveScheduler::Add(this); // add to scheduler

	Restart();
}

void Cdiscover::Restart()
{
	Cancel();
	release();

	User::LeaveIfError(timer.CreateLocal()); timer_is_open=true;
#ifndef __WINS__
	User::LeaveIfError(socket_server.Connect()); socket_server_is_open=true;
	TProtocolName aName=_L("BTLinkManager");
	User::LeaveIfError(socket_server.FindProtocol(aName, pInfo));
	User::LeaveIfError(hr.Open(socket_server,pInfo.iAddrFamily,pInfo.iProtocol)); hr_is_open=true;
#endif

	iInterval=5*60;
	Settings().GetSettingL(SETTING_BT_SCAN_INTERVAL, iInterval);
	TTimeIntervalMicroSeconds32 w(iInterval*1000*1000);
	timer.After(iStatus, w);
	SetActive();
	current_state=IDLE;
}

Cdiscover* Cdiscover::NewL(MApp_context& Context)
{
	CALLSTACKITEM(_L("Cdiscover::NewL"));

	Cdiscover* ret=new (ELeave) Cdiscover(Context);
	CleanupStack::PushL(ret);
	ret->ConstructL();
	CleanupStack::Pop();
	return ret;
}

void Cdiscover::release()
{	
	CALLSTACKITEM(_L("Cdiscover::release"));

	if (timer_is_open) timer.Close();
	timer_is_open=false;
	if (hr_is_open) hr.Close();
	hr_is_open=false;
	if (socket_server_is_open) socket_server.Close();
	socket_server_is_open=false;
}

Cdiscover::~Cdiscover()
{
	CALLSTACKITEM(_L("Cdiscover::~Cdiscover"));

	Cancel();
	release();
}	

void Cdiscover::DoCancel()
{
	CALLSTACKITEM(_L("Cdiscover::DoCancel"));

	switch(current_state) {
	case IDLE:
		timer.Cancel();
		break;
	default:
		hr.Cancel();
		break;
	}
}

const TDesC& Cdiscover::get_value()
{
	CALLSTACKITEM(_L("Cdiscover::get_value"));

	return names;
}


void Cdiscover::CheckedRunL()
{
	CALLSTACKITEM(_L("Cdiscover::CheckedRunL"));

	TBuf<30> msg;
	if (iStatus.Int()!=KErrNone && iStatus.Int()!=KErrHostResNoMoreResults ) {
		_LIT(stat_err, "error: %d");
		msg.Format(stat_err, iStatus.Int());
		post_new_value(msg, Mlogger::ERR);
		Restart();
		return;
	}
#ifndef __WINS__
	switch(current_state) {
	case FIRST_SEARCH:
		if (iStatus.Int()!=KErrHostResNoMoreResults) {
			names.Zero();
			TBuf<30> addr;
			TBTSockAddr btaddr(name().iAddr);
			btaddr.BTAddr().GetReadable(addr);
			names.Append(addr);
			names.Append(_L(" ["));
			names.Append(name().iName);
			names.Append(_L("] "));
			hr.Next(name, iStatus);
			SetActive();
			current_state=NEXT_SEARCH;
		} else {
			if (prev_env.Length()>0) {
				post_new_value(_L(""));
				prev_env.Zero();
			}
			current_state=IDLE;
			TTimeIntervalMicroSeconds32 w(iInterval*1000*1000);
			timer.After(iStatus, w);
			current_state=IDLE;
			SetActive();
		}

		break;
	case NEXT_SEARCH:
		if (iStatus.Int()!=KErrHostResNoMoreResults) {
			if (! (names.Length()+32+name().iName.Length() > names.MaxLength()) ) {
				TBuf<30> addr;
				TBTSockAddr btaddr(name().iAddr);
				btaddr.BTAddr().GetReadable(addr);
				names.Append(addr);
				names.Append(_L(" ["));
				names.Append(name().iName);
				names.Append(_L("] "));
			}
			hr.Next(name, iStatus);
			SetActive();
		} else {
			if (prev_env.Compare(names)) {
				post_new_value(names);
				prev_env=names;
			}
			TTimeIntervalMicroSeconds32 w(iInterval*1000*1000);
			timer.After(iStatus, w);
			current_state=IDLE;
			SetActive();
		}
		break;
	case IDLE:
		addr.SetIAC(KGIAC);
		addr.SetAction(KHostResInquiry|KHostResName|KHostResIgnoreCache);
		hr.GetByAddress(addr, name, iStatus);

		SetActive();
		
		current_state=FIRST_SEARCH;
		break;
	}
#else
	post_new_value(_L("0020e04c71b8 [GLOMGOLD-25] 0002ee51c437 [Reno7650]"));

#endif

}
